require("phase.lua")


phase.game = {}

clnt = {}
clnt.planet = {}
clnt.ship = {}
clnt.selection = {}

server = {}
server.planet = {}

function sound(title)
	if audio.playing == false then
		love.audio.play(title)
	end
	audio.playing = true
end

function phase.game:load()
	-- Server related variables for the client
	_player = 1
	
	-- User related things for the client
	percent = 25
	selection = {}
	
	-- Game related things for the client
	ships = {}
	colliding = {}
	
	particles = {}
	
	atmosphere = love.graphics.newImage("atmosphere.png")
end

local	font12 = love.graphics.newFont(love.default_font, 12)
local	font18 = love.graphics.newFont(love.default_font, 18)


-- AI BY OSUF
function phase.game:doAIFun(player, dt, defenseRate)
	function sendShips(from, to, amount)
		amount = amount or math.floor(planets[from].troops)
		amount = math.min(amount, 600 - bodyCount)
		if amount <= 0 then return; end
		for i=1,amount do
			createship(from, to, amount)
		end
	end

	defenseRate = defenseRate or 0.5
	local	minAttackForceRate = 1.2
	
	local	src = math.random(1, #planets)
	if planets[src].owner ~= player then return; end
	
	-- Pick a random planet and evaluate whether we should attack or support it.
	local	trg= math.random(1, #planets)
	if src == trg then return; end

	local	srcShips = planets[src].troops
	if not (planets[trg].owner == player or planets[trg].owner == neutral) then
		local appraison = math.random(0.67, 1.42)
	end
	local	trgShips = planets[trg].troops * (appraison or 1)
	
	-- Add the ship to the count of the planet it is heading towards.
	local	totalShips = {}
	for i, v in pairs(ships) do
		if v.d == src then
			if v.owner ~= planets[v.d].owner then
				srcShips = srcShips - 1
			else
				srcShips = srcShips + 1
			end
		elseif v.d == trg then
			if v.owner == planets[v.d].owner then
				trgShips = trgShips + 1
			elseif v.owner == player then
				trgShips = trgShips - 1
			elseif planets[trg].owner == player then
				trgShips = trgShips - 1
			else
				trgShips = trgShips + 1 -- enemy ships in the way when attacking.
			end
		end
	end
	srcShips = math.max(0, srcShips)
	trgShips = math.max(0, trgShips)
	
	local	srcRat = srcShips / (1 + planets[src].size + planets[src].population)
	local	trgRat = trgShips / (1 + planets[trg].size + planets[trg].population)
	if planets[trg].owner == player then
		-- My planet. Send half of the respective excess of ratios.
		local	sendRat = (srcRat - trgRat) / 2
		if sendRat > 0 then
			sendShips(src, trg, math.floor(sendRat * (1+planets[src].size + planets[src].population)))
		else
			sendShips(trg, src, math.floor(-sendRat * (1+planets[trg].size + planets[trg].population)))
		end
	elseif srcRat > defenseRate then -- we want at least defenseRate * size of planet to stay
		local	planetValue = planets[trg].size + planets[trg].population
		local	dx = planets[trg].x - planets[src].x
		local	dy = planets[trg].y - planets[src].y
		local	d = (dx^2+dy^2)^0.5
		-- A planet on the far end of the galaxy is about a fifth
		-- as much worth as one a tenth of a screen from you
		-- (according to this code).
		planetValue = planetValue / (3*d)
		-- more prone to attack "valuable" planets
		if math.random() < planetValue^2 then
			local	myTotalExcess = 0
			for k,v in pairs(planets) do
				if v.owner == player then
					myTotalExcess = myTotalExcess + math.max(0, v.troops - v.size * defenseRate)
				end
			end
			local	neededShips = math.floor(minAttackForceRate * trgShips + planets[trg].size * defenseRate / 2)
			if myTotalExcess >= neededShips then
				if planets[src].troops > planets[src].size * defenseRate then
					sendShips(src, trg, math.floor(math.min(neededShips, planets[src].troops - planets[src].size * defenseRate)))
					neededShips = neededShips - math.floor(planets[src].troops - planets[src].size * defenseRate)
				end
				for k,v in pairs(planets) do
					if k ~= src and v.owner == player and v.troops > v.size * defenseRate then
						sendShips(k, trg, math.floor(math.min(neededShips, v.troops - v.size * defenseRate)))
						neededShips = neededShips - math.floor(v.troops - v.size * defenseRate)
					end
				end
			end
		end
	end
end


-- ############
-- DRAW FUNCTION
-- ############
-- This file is very large. Perhaps create some functions for objects, e.g. a planet.draw(planet), ship.update(ship, dt).
-- If you do this and you store a ship in a table 't', then you can write setmetatable(t, ship) and then use ship:update(dt).
function phase.game:draw()
	--OSUF set font
	love.graphics.setFont(font12)

	love.graphics.setColorMode(love.color_modulate) -- adds colors to particles
		love.graphics.setBlendMode(love.blend_additive)
	
	
	-- OBJECTS
	for i, v in pairs(planets) do -- PLANETS
		local radius = v.shape:getRadius()
		-- takeover particles
		love.graphics.draw(v.particles.takeover, 0, 0)
		-- for the colored atmosphere
		local atmoshpereRadius = radius*0.05
		local color = player[v.owner].color
		color:setAlpha(255)
		love.graphics.setColor(color)
		love.graphics.draw(atmosphere, v.body:getX(), v.body:getY(), 0, atmoshpereRadius, atmoshpereRadius)
		color:setAlpha(255)
		-- for troop numbers
		love.graphics.setColor(white)
		if v.owner == _player or v.owner == 0 then
			love.graphics.drawf(math.floor(v.troops), v.body:getX(), v.body:getY() + love.graphics.getFont():getHeight()/3, 0, love.align_center)
		end
		-- for population particles
		love.graphics.setColor(white)
		love.graphics.draw(v.particles.population, 0, 0)
	end
	
	love.graphics.setBlendMode(love.blend_additive) -- makes things look better (ships, selection rings, etc)
		
	for i, v in pairs(ships) do -- SHIPS
		if v ~= nil then
			local o = v.owner
			love.graphics.setColor(player[o].color)
			love.graphics.setLine(1, love.line_smooth)
			love.graphics.circle(love.draw_line, v.body:getX(), v.body:getY(), v.shape:getRadius())
		end
	end
	
	
	-- OTHERS
	for i, v in pairs(particles) do -- PARTICLES
		love.graphics.draw(v, 0, 0)
	end
	
	for i, v in pairs(selection) do -- SELECTION RINGS
		love.graphics.setColor(clr(255, 255, 255, 192))
		love.graphics.setLine(2, love.line_smooth)
		local radius = planets[i].shape:getRadius() + 3
		love.graphics.circle(love.draw_line, planets[i].body:getX(), planets[i].body:getY(), radius, radius*2)
	end
	
	
	-- MOUSE
	love.graphics.setColor(clr(255, 255, 255, 80)) -- MOUSE POSITION - crosshairs
	love.graphics.setLine(1, love.line_smooth)
	local mx, my = love.mouse.getPosition()
	love.graphics.line(mx, 0, mx, love.graphics.getHeight())
	love.graphics.line(0, my, love.graphics.getWidth(), my)
	if pressed == true then -- PRESSED POSITION | SELECTION AREA that appears when holding down the left mouse key
		love.graphics.line(pressedX, 0, pressedX, love.graphics.getHeight())
		love.graphics.line(0, pressedY, love.graphics.getWidth(), pressedY)
		love.graphics.setColor(clr(255, 255, 255, 192))
		love.graphics.line(mx, my, mx, pressedY)
		love.graphics.line(mx, my, pressedX, my)
		love.graphics.line(pressedX, pressedY, mx, pressedY)
		love.graphics.line(pressedX, pressedY, pressedX, my)
	end
	
	love.graphics.setFont(font18)
	if m == true then-- MOUSEOVER OUTPUT (hover)
		local typ = mouse.typ
		local pid = mouse.i
		if not (pressed or shift or ctrl) then -- DRAW LINES
			for i, v in pairs(selection) do
				local dx = typ[pid].body:getX() - planets[i].body:getX()
				local dy = typ[pid].body:getY() - planets[i].body:getY()
				--OSUF Check that d is not 0!
				local d = (dx^2 + dy^2)^0.5
				local vx = dx / d
				local vy = dy / d
				
				love.graphics.setColor(clr(255, 255, 255, 192))
				love.graphics.setLine(2, love.line_smooth)
				local srcx = planets[i].body:getX() + vx * (planets[i].shape:getRadius() + 2)
				local srcy = planets[i].body:getY() + vy * (planets[i].shape:getRadius() + 2)
				local trgx = typ[pid].body:getX() - vx * (typ[pid].shape:getRadius() + 2)
				local trgy = typ[pid].body:getY() - vy * (typ[pid].shape:getRadius() + 2)
				love.graphics.line(srcx, srcy, trgx, trgy)
			end
		end
		if beep ~= pid then
			beep = pid
			-- love.audio.play(audio.sound.selection.hover)
		end
		love.graphics.setColor(clr(255, 255, 255, 204))
		love.graphics.setLine(2, love.line_smooth)
		love.graphics.circle(love.draw_line, typ[pid].body:getX(), typ[pid].body:getY(), typ[pid].shape:getRadius() + 3, 40)
		--OSUF Newline, scale
		love.graphics.drawf("Owner: "..player[typ[pid].owner].nick .."\nOutput: "..typ[pid].size.."+"..math.floor(typ[pid].population).."="..typ[pid].size+math.floor(typ[pid].population), 100, 10, 300, love.align_left, 0.5)
	end
	
	
	-- TEXT
	love.graphics.setColor(clr(255, 255, 255, 192))
	local output = "Ships: "..percent.."%"
	love.graphics.drawf(output,love.graphics.getWidth() - love.graphics.getFont():getWidth(output) - love.graphics.getFont():getHeight(), love.graphics.getHeight() - love.graphics.getFont():getHeight(), love.graphics.getFont():getWidth(output))
end


-- ############
--           CLIENT
-- ############

-- PLANETS
-- Particles
--   Population
--   Update particles
-- Interface
--   Update mouseover
--   Population increases
function clnt.planet:update(i, v, dt)
-- PARTICLES
	-- POPULATION
	local cosinus, sinus = v.body:getPosition()
	cosinus, sinus = getCamera():pos(cosinus, sinus)
	
	local angular = math.random(0, 359)
	local radius = math.random(v.shape:getRadius() * 0.15, v.shape:getRadius() * 0.8) * getCamera():getScaleFactor()
	cosinus = cosinus + radius * math.cos(math.rad(angular))
	sinus = sinus + radius * math.sin(math.rad(angular))
	v.particles.population:setPosition(cosinus, sinus)
	v.particles.population:setEmissionRate((v.shape:getRadius()^2*math.pi) * v.population/v.size * 0.1)
	
	-- UPDATE PLANETS PARTICLES
	v.particles.takeover:update(dt)
	v.particles.population:update(dt)
	
-- INTERFACE
	-- UPDATE MOUSEOVER PLANET
	local select = v.shape:testPoint(mouseX, mouseY)
	if select == true and m == false then
		mouse = {typ = planets, i = i}
		m = true
	end
	
	-- PLANETS POPULATION INCREASES
	v.population = v.population + (v.size - math.floor(v.population)) * dt * settings.growthRate / 100
end

-- SHIPS
-- Create bodies and shapes
-- Move ships around
function clnt.ship:update(i, v, dt)
	-- CREATE BODIES AND SHAPES FOR SHIPS
	if not v.body then -- controlls whether the selected ship already has a body
		local body = love.physics.newBody(world, v.x, v.y)
		local shape = love.physics.newCircleShape(body, 2)
		shape:setCategory(2)
		--shape:setMask(2)
		--shape:setIndex(v.d)
		--OSUF could also add body=body, ship=v
		shape:setData{typ = "ship", i = i}
		body:setMassFromShapes()
		v.body = body
		v.shape = shape
	end
	
	-- MOVE SHIPS AROUND
	if v.d ~= nil then
		local dx = planets[v.d].body:getX() - v.body:getX()
		local dy = planets[v.d].body:getY() - v.body:getY()
		local s = settings.speedMultiplier -- SPEEDMULTIPLIER
		local d = (dx^2 + dy^2)^0.5
		local vx = dx / d
		local sx = vx * s
		local vy = dy / d
		local sy = vy * s
		
		v.body:setVelocity(sx, sy)
	end
end

-- SELECTION
function clnt.selection:update(i, v)
	if planets[i].owner ~= _player then
		table.remove(selection, i)
	end
end


-- ############
--           SERVER
-- ############

-- PLANETS
-- Increase troopnumber
function server.planet:update(i, v, dt)
	-- PLANETS PRODUCE SHIPS
		if v.owner ~= 0 then -- planet must not be neutral
			v.troops = v.troops + (v.size + math.floor(v.population)) * dt / settings.productionCost -- increase production cost
		end
end


-- ############
--   UPDATE PROCESS
-- ############
function phase.game:update(dt)
	world:update(dt*_speed/100)
	

	-- mouseover beep (m = true -> beep)
	mouseX, mouseY = love.mouse.getPosition() 
	m = false
-- UPDATE PLANETS
	for i, v in pairs(planets) do
		clnt.planet:update(i, v, dt)
		server.planet:update(i, v, dt)
	end
	-- mouseover beep beepsreseter
	if m == false then
		beep = 0
	end
	
--RUN OSUFs AI
	for i, v in pairs(player) do
		if v.nick == "CPU" then
			phase.game:doAIFun(i, dt, 0.25)
		end
	end
	
-- UPDATE SHIPS
	for i, v in pairs(ships) do
		clnt.ship:update(i, v, dt)
	end
	
-- UPDATE SELECTION
	for i, v in pairs(selection) do
		clnt.selection:update(i, v)
	end


-- COLLISIONS BETWEEN PLANETS AND SHIPS: colliding[planetID][shipID]
	for i in pairs(colliding) do
		local pid = i
		for i in pairs(colliding[pid]) do
			local sid = i
			if planets[pid].owner == ships[sid].owner then -- Ship owner == Planet owner
				planets[pid].troops = planets[pid].troops + 1
			else -- Ship is an enemy
				if planets[pid].troops >= 1 then
					local x, y = colliding[pid][sid]:getPosition()
					--OSUF Could also use getCamera:pos(x,y) for this. That would also account for maps that are scrollable
					x, y = getCamera():pos(x,y)
					particles.collision:setPosition(x, y)
					particles.collision:start()
					love.audio.play(audio.sound.collision.hit)
					planets[pid].troops = planets[pid].troops - 1
					planets[pid].population = planets[pid].population * 0.99 -- planet looses 1%
				else -- Change planetowner
					planets[pid].particles.takeover:setColor(player[ships[sid].owner].color, clr(0, 0, 0, 128))
					planets[pid].particles.takeover:start()
					love.audio.play(audio.sound.collision.takeover)
					planets[pid].owner = ships[sid].owner
					planets[pid].troops = 1
					planets[pid].population = planets[pid].population * 0.50 -- planet looses 50%
				end
			end
			assert(assert(ships[sid], "wrong id ("..sid..")").shape, "shape doesn't exist"):destroy()
			ships[sid].body:destroy()
			ships[sid] = nil
			colliding[pid][sid] = nil
		end
		colliding[pid] = nil
	end
	
	for i, v in pairs(particles) do
		v:update(dt)
	end
	
	if love.keyboard.isDown(love.key_space) then
		for i, v in pairs(planets) do
			if v.owner == _player then
			selection[i] = true
			end
		end
	end
end


-- ############
-- COLLISION FUNCTION
-- ships collide with objects
-- ############
function phase.game.collision(a, b, c)
	if a.typ == "planet" and b.typ == "ship" and ships[b.i].d == a.i then -- a = Planet; b = Ship
		--OSUF colliding[a.i] = colliding[a.i] or {}
		if colliding[a.i] == nil then
			colliding[a.i] = {}
		end
		colliding[a.i][b.i] = c
	end
	
	if b.typ == "planet" and a.typ == "ship" and ships[a.i].d == b.i then -- b = Planet; a = Schiff
		if colliding[b.i] == nil then
			colliding[b.i] = {}
		end
		colliding[b.i][a.i] = c
	end
end


-- ############
--        KEYBOARD
-- ############
function phase.game:keypressed(key)
	-- SPEED
	-- Until now, it only affects the rate at which the world is upgraded.
	-- That means, if ships would use applyImpulse instead of setVelocity,
	-- they would have a velocity of 1000000..., if the game was paused for an hour or so.
	
	if key == love.key_kp_multiply then _speed = _speed * 1.1 end
	if key == love.key_kp_divide then _speed = _speed / 1.1 end
	if key == love.key_kp_plus then _speed = _speed + 10 end
	if key == love.key_kp_minus then _speed = _speed - 10 end
	
	if key == love.key_kp_period then _speed = 100 end
	if key == love.key_kp_enter then  end
	
	if key == love.key_kp0 then _speed = 0 end
	if key == love.key_kp1 then _speed = 100 end
	if key == love.key_kp2 then _speed = 200 end
	if key == love.key_kp3 then _speed = 300 end
	if key == love.key_kp4 then _speed = 400 end
end


-- ############
-- MOUSE BUTTON PRESSED
-- ############
function phase.game:mousepressed(x, y, button)
	if button == love.mouse_left then
		pressed = true
		pressedX = x
		pressedY = y
	end
end

-- ############
-- MOUSE BUTTON RELEASED
-- ############
function phase.game:mousereleased(x, y, button)

	-- PERCENT OF SHIPS SENT
	if button == love.mouse_wheelup and percent <= 95 then percent = percent + 5 end -- PERCENT + 5
	if button == love.mouse_wheelup and alt == true then percent = 100 end -- 100
	if button == love.mouse_wheeldown and percent >= 10 then percent = percent - 5 end -- PERCENT - 5
	if button == love.mouse_wheeldown and alt == true then percent = 5 end -- 5

	
	-- CLEAR SELECTION IF NOT SHIFT/CTRL
	if shift == false and ctrl == false and button == love.mouse_left then selection = {} end
	
	
	-- LEFT MOUSE: selection
	if button == love.mouse_left then
		audio.playing = false
		for i, v in pairs(planets) do
			if v.owner == _player then
				local selectA = v.shape:testPoint (pressedX, pressedY)
				local selectZ = v.shape:testPoint(x, y)
				if selectA == true and selectZ == true then -- SELECT ONE
					if shift == true and selection[i] == nil then
						sound(audio.sound.selection.add)
						selection[i] = true
						
					elseif ctrl == true and selection[i] == true then
						sound(audio.sound.selection.remove)
						selection[i] = nil
						
					elseif ctrl == false and shift == false then
						sound(audio.sound.selection.select)
						selection[i] = true
						
					end
				end
				
				if ((pressedX < v.body:getX() and v.body:getX() < x) or (pressedX > v.body:getX() and v.body:getX() > x)) and ((pressedY < v.body:getY() and v.body:getY() < y) or (pressedY > v.body:getY() and v.body:getY() > y)) then
					if ctrl == false and shift == true and selection[i] == nil then
						sound(audio.sound.selection.add)
						selection[i] = true
						
						
					elseif ctrl == true and shift == false and selection[i] == true then
						sound(audio.sound.selection.remove)
						selection[i] = nil
						
					elseif ctrl == true and shift == true then
						if selection[i] == nil then selection[i] = true 
						else selection[i] = nil end
						
					elseif ctrl == false and shift == false then
						sound(audio.sound.selection.select)
						selection[i] = true
					end
				end
			end
		end
		if selection == {} then -- LEFT MOUSE: unselect
			love.audio.play(audio.sound.selection.clear)
		end
	end
	
	
	pressed = false
	
	
	-- RIGHT MOUSE: target
	if button == love.mouse_right and selection ~= {} and ctrl == false and shift == false then
		for i, v in pairs(planets) do
			local select = v.shape:testPoint(x, y)
			if select == true then -- TARGET PLANET
				target = i
				
				for i in pairs(selection) do
					if planets[i].owner == _player and i ~= target then
						local order = math.floor(math.floor(planets[i].troops) * percent / 100 + 0.5)
						
						--OSUF How about keeping a "target" for each planet and a counter for the number of ships to send?
						-- Every x seconds, send one ship out until the orderis filled.
						-- This also makes it very easy for an AI to send ships.
						for troops=1, order do
							createship(i, target, order)
						end
					end
					i = nil
				end
			end
		end
	end
end


function createship(source, destination, amount)
	if planets[source].troops >= 1 then
		local dx = planets[destination].body:getX() - planets[source].body:getX()
		local dy = planets[destination].body:getY() - planets[source].body:getY()
		local d = (dx^2 + dy^2)^0.5
		local vx = dx / d * planets[i].shape:getRadius()
		local vy = dy / d * planets[i].shape:getRadius()
							
		local r = amount^0.70
		
		local x = planets[source].body:getX() + 2*vx + math.random(- r, r)
		local y = planets[source].body:getY() + 2*vy + math.random(- r, r)
		local owner = planets[source].owner
		
		table.insert(ships, {x = x, y = y, d = destination, owner = owner})
		
		planets[source].troops = planets[source].troops - 1
	end
end
