--[[   Project started on 08. Feb 2009
          offline-Version of 26. Sep 2009

Copyright 2009 Daniel Sturm <skasski@gmx.at> ]]



-- AUDIO
love.audio.setChannels(12)
audio = {}

-- SETTINGS
audio.settings = {}
audio.settings.play = "on"

-- MUSIC
audio.music = {}
-- intro - played while in the menu
-- audio.music.intro = love.audio.newMusic("audio/Silence - Expirience.ogg")

-- playing - played while playing
audio.music.playing = {}
audio.music.playing[1] = love.audio.newMusic("audio/Silence - Larmes.ogg")
audio.music.playing[2] = love.audio.newMusic("audio/Silence - Cellule.ogg")
audio.music.playing[3] = love.audio.newMusic("audio/Silence - Rialiti.ogg")
audio.music.playing[4] = love.audio.newMusic("audio/Silence - Follow me.ogg")

-- long - played in long games (?)
-- audio.music.long = love.audio.newMusic("audio/Silence - Effaci.ogg")

-- credits - played when showing a list of contributors
-- audio.music.credits = love.audio.newMusic("audio/Silence - Sale.ogg")

-- SOUND
audio.sound = {}
-- played on collisions
audio.sound.collision = {}
audio.sound.collision.land = nil
audio.sound.collision.hit = love.audio.newSound("audio/collision/hit.ogg")
audio.sound.collision.hit:setVolume(0.225)
audio.sound.collision.takeover = love.audio.newSound("audio/collision/takeover.ogg")
audio.sound.collision.takeover:setVolume(0.25)

-- played when selecting something
audio.sound.selection = {}
audio.sound.selection.hover = nil
audio.sound.selection.select = love.audio.newSound("audio/selection/select.ogg")
audio.sound.selection.select:setVolume(0.5)
audio.sound.selection.add = love.audio.newSound("audio/selection/add.ogg")
audio.sound.selection.add:setVolume(0.25)
audio.sound.selection.remove = love.audio.newSound("audio/selection/remove.ogg")
audio.sound.selection.remove:setVolume(0.25)

math.randomseed(os.time())
local randomizer = math.random(100)

require("setup.lua")
require("game.lua")
require("camera.lua")
setCamera(camera.stretchToResolution(400, 300))
--_multiplier = getCamera():scaleX(1) -- used for particles position, etc.
_speed = 100
_counter = 0


function load()
	clr = function(r, g, b, a) return love.graphics.newColor(r, g, b, a) end
	
	-- Keyboard
	shift = false
	ctrl = false
	alt = false
	space = false
	-- Mouse
	pressed = false
	
	
	bodyCount = 0
	
	part1 = love.graphics.newImage("part1.png");
	phase.game:load()
	setup:load()
end

local	font18 = love.graphics.newFont(love.default_font, 20/getCamera():getScaleFactor())

function draw()
	phase.game:draw()
	local	objectCount = "Number of objects:\n" ..bodyCount.. "\n(more than 1'000\nmay crash the game)\nPlayers: "..playerCount.."\nPlanets: "..planetCount.."\nShips: "..shipCount
	love.graphics.setFont(font18)
	love.graphics.draw(objectCount, love.graphics.getWidth()*0.75, 50, 0, 0.5)
	love.graphics.draw("Speed: ".._speed.."\nRefreshing interval:\n"..love.timer.getDelta().."\nFPS: "..love.timer.getFPS().."\nCounter: ".._counter, love.graphics.getWidth()*0.75, 200, 0, 0.5)
	love.graphics.draw("Music: "..audio.settings.play, love.graphics.getWidth()*0.75, love.graphics.getHeight()*0.1)
end

function update(dt)
	_counter = _counter + dt
	
	playerCount = playernumber or 0
	planetCount = table.getn(planets) or 0
	shipCount = table.getn(ships) or 0
	
	phase.game:update(dt)
	
	bodyCount = world:getBodyCount()
	
	for i, v in pairs(planets) do
		if v.owner > 0 then
			shipCount = shipCount + math.floor(v.troops)
		end
	end
	if not love.audio.isPlaying() and audio.settings.play == "on" then
		local rand = math.random(1, table.getn(audio.music.playing))
		if last ~= rand then
			love.audio.play(audio.music.playing[rand])
			last = rand
		end
	end
	
	
	if love.keyboard.isDown(love.key_lshift) or love.keyboard.isDown(love.key_rshift) then  shift = true   else   shift = false   end
	if love.keyboard.isDown(love.key_lctrl)  or love.keyboard.isDown(love.key_rctrl)  then   ctrl = true   else    ctrl = false   end
	if love.keyboard.isDown(love.key_lalt)   or love.keyboard.isDown(love.key_ralt)   then   alt = true   else    alt = false   end
	
	
end


function keypressed(key) phase.game:keypressed(key)
	if key == love.key_backspace then love.system.restart() end -- BACKSPACE: restart system
	if key == love.key_escape    then love.system.exit() end    -- ESCAPE:        exit system
	if     key == love.key_f9
		and audio.settings.play == "on" then audio.settings.play = "off" love.audio.stop()
	elseif key == love.key_f9           then audio.settings.play = "on" end
end
function mousepressed(x, y, button) phase.game:mousepressed(x, y, button) end
function mousereleased(x, y, button) phase.game:mousereleased(x, y, button) end


camera.lateInit()