setup = {}

-- FUNCTION USED TO CREATE PLANETS AND THEIR PARTICLES
function createplanet(x, y, typ, size, population, troops, owner)
	table.insert(planets, {x = x, y = y, typ = typ, size = size, population = population, troops = troops, owner = owner})
	i = table.getn(planets)
	v = planets[i]
	
	local body = love.physics.newBody(world, x, y, 0)
	local shape = love.physics.newCircleShape(body, 5 + size / 15)
	-- setData{planet = v}
	shape:setData({typ = "planet", i = i})
	shape:setCategory(1) 
	shape:setFriction(1)
	shape:setRestitution(0.75)
	v.body = body
	v.shape = shape
	
	local x, y = v.body:getPosition()
	x, y = getCamera():pos(x,y)
	local scale = getCamera():getScaleFactor()
	
	v.particles = {}
	
	local p = love.graphics.newParticleSystem(part1, 1000)
	p:setEmissionRate(v.shape:getRadius() * 30)
	p:setPosition(x, y)
	p:setSpeed(40, 180)
	p:setSize(2, 0.5)
	p:setLifetime(0.5)
	p:setParticleLife(0.20, 0.75)
	p:setDirection(0)
	p:setSpread(360)
	p:setTangentialAcceleration(-100, 100)
	p:stop()
	v.particles.takeover = p
	
	local p = love.graphics.newParticleSystem(part1, 1000)
	p:setColor(clr(255, 255, 255, 255))
	p:setSpeed(v.shape:getRadius()/80*scale, v.shape:getRadius()/40*scale)
	p:setSize(0.025*scale)
	p:setLifetime(-1)
	p:setParticleLife(2, 5)
	p:setDirection(0)
	p:setSpread(360)
	p:setTangentialAcceleration(v.shape:getRadius()*-0.05, v.shape:getRadius()*0.05)
	p:start()
	v.particles.population = p
end

--OSUF this seems a lot like a loading of phase.game
--OSUF a setup only needs to keep track of players and planets and their properties
function setup:load()
	-- ############
	--    GAME SETTINGS
	-- ############
	settings = {}
	
	settings.worldsize = 300
	
	-- PRODUCTION COST determines the planet output needed to produce a single ship.
	-- The planet output is calculated by its size and its population.
	-- should be a number from 100 (fast production) to 400 (slow production)
	settings.productionCost = 400
	
	-- SPEED MULTIPLIER affects the speed of ships.
	-- should be a number from 20 (slow) to 60 (fast)
	settings.speedMultiplier = 40
	
	-- GROWTH RATE of planets population
	-- should be a number from 1 (slow growth) to 5 (fast growth)
	settings.growthRate = 2.5
	
	-- NUMBER OF PLAYERS
	-- may range from 2 (one enemy) to 6 (five enemies)
	-- nil or 0 lets the game generate a random number
	settings.playerNumber = nil
	
	-- NUMBER OF PLANETS FOR MAPGENERATION
	-- may range from 2 to 100
	-- nil or 0 lets the game generate a random number
	settings.planetNumber = nil
	
	
	setCamera(camera.stretchToResolution(settings.worldsize*1.33, settings.worldsize))

	particles = {}
	local scale = getCamera():getScaleFactor()
	
	
	local p = love.graphics.newParticleSystem(part1, 1000)
	p:setEmissionRate(200)
	p:setSpeed(0, 20)
	p:setSize(0.5, 0.25)
	p:setColor(clr(255, 128, 64, 255), clr(255, 0, 0, 128))
	p:setLifetime(0.1)
	p:setParticleLife(0.75, 1.5)
	p:setDirection(0)
	p:setSpread(360)
	p:setTangentialAcceleration(-50, 50)
	p:stop()
	particles.collision = p

	local margin = 200
	local worldsizes = settings.worldsize + margin*2
	world = love.physics.newWorld(-margin, -margin, worldsizes, worldsizes, 0, 0, false)
	world:setCallback(phase.game.collision)
	
	white = love.graphics.newColor(255, 255, 255)
	
	
	-- START
	
	clr = function(r, g, b, a) return love.graphics.newColor(r, g, b, a) end
	
	-- Keyboard
	shift = false
	ctrl = false
	alt = false
	space = false
	-- Mouse
	pressed = false
	
--PLAYERS
-- [0] = neutral
	player = {}
	player[0] = {nick = "none", color = love.graphics.newColor(128, 128, 128, 255)}-- N,  grey
	player[1] = {nick = "Player", color = love.graphics.newColor(255, 156, 0, 255)}-- P1,  orange
	player[2] = {nick = "CPU", color = love.graphics.newColor(0, 156, 255, 255)}   -- P2,  blue
	player[3] = {nick = "CPU", color = love.graphics.newColor(156, 0, 255, 255)}   -- P3,  pink
	player[4] = {nick = "CPU", color = love.graphics.newColor(156, 255, 0, 255)}   -- P4,  green
	player[5] = {nick = "CPU", color = love.graphics.newColor(0, 200, 156, 255)}   -- P5,  teal
	player[6] = {nick = "CPU", color = love.graphics.newColor(255, 0, 156, 255)}   -- P6,  red
	
	
-- PLANETS
-- size represents the minimum production output as well as the maximum population of a planet
-- population increases the production output just like size does, but increases over time and gets killed on attacks
-- troops are ships located on the planet
-- owner is the index taken from the player that owns the planet (0 = neutral)
	planets = {}
	
	if playernumber then
		if playernumber > 6  then playernumber = 6   end
		if playernumber == 0 then playernumber = nil end
	end
	
	if planetnumber then
		if planetnumber > 0  then planetnumber = 100 end
		if planetnumber == 0 then planetnumber = nil end
	end
	
	playernumber = settings.playerNumber or math.random(2, 6)
	planetnumber = settings.planetNumber or math.random(playernumber*1.3, playernumber*2) * (settings.worldsize/200)^2
	
	-- ############
	-- PLANET CREATION
	-- ############
	
	-- PLANETS FOR PLAYERS
	local angular = math.random(0, 360)
	local radius = settings.worldsize/2 - 50
	for i=1, playernumber do
		cosinus = settings.worldsize/2 + radius * math.cos(math.rad(angular))
		sinus = settings.worldsize/2 + radius * math.sin(math.rad(angular))
		createplanet(cosinus, sinus, "terra", 100, 100, 100, i)
		angular = angular + 360/playernumber
	end
	
	tries = 0
	padding = 30
	-- NEUTRAL PLANETS
	while table.getn(planets) < planetnumber + playernumber do
		if tries > 100 then break end
		local x = math.random(padding, settings.worldsize - padding)
		local y = math.random(padding, settings.worldsize - padding)
		local size = math.random(10, 100)
		local population = math.random(0, size*0.5)
		local troops = math.random(0.05*size, 0.80*size)
		local create = true
		
		local types = {"terra", "stone", "water"}
		local typ = types[math.random(1, table.getn(types))]
		
		for i, v in pairs(planets) do
			local body = love.physics.newBody(world, v["x"], v["y"])
			local shape = love.physics.newCircleShape(body, 17.5+(v["size"]+size)/15)
			if shape:testPoint(x, y) == true then
				create = false
			end
			shape:destroy()
			body:destroy()
		end
		
		if create == true then
			createplanet(x, y, typ, size, population, troops, 0)
		end
		
		tries = tries + 1
	end
end
